@@ -64,6 +64,8 @@ class InstallTarget(object):
     # Whether to prefer user-specific installation
     user: Optional[bool]
 
+    prefix: Optional[str]
+
     # TODO(jelmer): Add information about target directory, layout, etc.
 
 
@@ -118,6 +120,11 @@ class Pear(BuildSystem):
 
     name = "pear"
 
+    PEAR_NAMESPACES = [
+        "http://pear.php.net/dtd/package-2.0",
+        "http://pear.php.net/dtd/package-2.1",
+        ]
+
     def __init__(self, path):
         self.path = path
 
@@ -146,10 +153,7 @@ class Pear(BuildSystem):
         try:
             root = xmlparse_simplify_namespaces(
                 path,
-                [
-                    "http://pear.php.net/dtd/package-2.0",
-                    "http://pear.php.net/dtd/package-2.1",
-                ],
+                self.PEAR_NAMESPACES
             )
         except ET.ParseError as e:
             logging.warning("Unable to parse package.xml: %s", e)
@@ -173,9 +177,25 @@ class Pear(BuildSystem):
 
     @classmethod
     def probe(cls, path):
-        if os.path.exists(os.path.join(path, "package.xml")):
-            logging.debug("Found package.xml, assuming pear package.")
-            return cls(os.path.join(path, "package.xml"))
+        package_xml_path = os.path.join(path, "package.xml")
+        if not os.path.exists(package_xml_path):
+            return
+
+        import xml.etree.ElementTree as ET
+        try:
+            tree = ET.iterparse(package_xml_path)
+        except ET.ParseError as e:
+            logging.warning("Unable to parse package.xml: %s", e)
+            return
+
+        if not tree.root:
+            # No root?
+            return
+
+        for ns in cls.PEAR_NAMESPACES:
+            if tree.root.tag == '{%s}package' % ns:
+                logging.debug("Found package.xml with namespace %s, assuming pear package.")
+                return cls(path)
 
 
 # run_setup, but setting __name__
@@ -221,7 +241,6 @@ import sys
 
 script_name = %(script_name)s
 
-save_argv = sys.argv.copy()
 g = {"__file__": script_name, "__name__": "__main__"}
 try:
     core._setup_stop_after = "init"
@@ -428,6 +447,8 @@ class SetupPy(BuildSystem):
             extra_args = []
             if install_target.user:
                 extra_args.append("--user")
+            if install_target.prefix:
+                extra_args.append("--prefix=%s" % install_target.prefix)
             self._run_setup(session, resolver, ["install"] + extra_args, fixers)
         else:
             raise NotImplementedError
@@ -442,7 +463,8 @@ class SetupPy(BuildSystem):
         if interpreter is None:
             interpreter = self.DEFAULT_PYTHON
         argv = [interpreter, "./setup.py"] + args
-        env = {}
+        # TODO(jelmer): Perhaps this should be additive?
+        env = dict(os.environ)
         # Inherit SETUPTOOLS_SCM_PRETEND_VERSION from the current environment
         if "SETUPTOOLS_SCM_PRETEND_VERSION" in os.environ:
             env["SETUPTOOLS_SCM_PRETEND_VERSION"] = os.environ[
@@ -522,14 +544,40 @@ class SetupPy(BuildSystem):
         if os.path.exists(os.path.join(path, "setup.py")):
             logging.debug("Found setup.py, assuming python project.")
             return cls(path)
-        if os.path.exists(os.path.join(path, "setup.cfg")):
-            logging.debug("Found setup.py, assuming python project.")
-            return cls(path)
         if os.path.exists(os.path.join(path, "pyproject.toml")):
             logging.debug("Found pyproject.toml, assuming python project.")
             return cls(path)
 
 
+class Bazel(BuildSystem):
+
+    name = "bazel"
+
+    def __init__(self, path):
+        self.path = path
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.path)
+
+    @classmethod
+    def exists(cls, path):
+        if not os.path.exists(os.path.join(path, "BUILD")):
+            return False
+        return True
+
+    @classmethod
+    def probe(cls, path):
+        if cls.exists(path):
+            logging.debug("Found BUILD, assuming bazel package.")
+            return cls(path)
+
+    def build(self, session, resolver, fixers):
+        run_with_build_fixers(session, ["bazel", "build", "//..."], fixers)
+
+    def test(self, session, resolver, fixers):
+        run_with_build_fixers(session, ["bazel", "test", "//..."], fixers)
+
+
 class Octave(BuildSystem):
 
     name = "octave"
@@ -678,6 +726,7 @@ class R(BuildSystem):
         return dc.copy_single(target_directory)
 
     def install(self, session, resolver, fixers, install_target):
+        extra_args.append("--prefix=%s" % install_target.prefix)
         r_path = guaranteed_which(session, resolver, "R")
         run_with_build_fixers(session, [r_path, "CMD", "INSTALL", "."], fixers)
 
@@ -713,6 +762,10 @@ class R(BuildSystem):
         if "Imports" in description:
             for s in parse_list(description["Imports"]):
                 yield "build", RPackageRequirement.from_str(s)
+        if "LinkingTo" in description:
+            for s in parse_list(description["LinkingTo"]):
+                yield "build", RPackageRequirement.from_str(s)
+        # TODO(jelmer): Suggests
 
     def get_declared_outputs(self, session, fixers=None):
         description = self._read_description()
@@ -732,7 +785,7 @@ class Meson(BuildSystem):
 
     def _setup(self, session, fixers):
         if not session.exists("build"):
-            session.check_call(["mkdir", "build"])
+            session.mkdir("build")
         run_with_build_fixers(session, ["meson", "setup", "build"], fixers)
 
     def clean(self, session, resolver, fixers):
@@ -746,7 +799,12 @@ class Meson(BuildSystem):
     def dist(self, session, resolver, fixers, target_directory, quiet=False):
         self._setup(session, fixers)
         with DistCatcher([session.external_path("build/meson-dist")]) as dc:
-            run_with_build_fixers(session, ["ninja", "-C", "build", "dist"], fixers)
+            try:
+                run_with_build_fixers(session, ["ninja", "-C", "build", "dist"], fixers)
+            except UnidentifiedError as e:
+                if "ninja: error: unknown target 'dist', did you mean 'dino'?" in e.lines:
+                    raise NotImplementedError
+                raise
         return dc.copy_single(target_directory)
 
     def test(self, session, resolver, fixers):
@@ -947,9 +1005,10 @@ class DistZilla(BuildSystem):
             return cls(os.path.join(path, "dist.ini"))
 
     def get_declared_dependencies(self, session, fixers=None):
-        lines = run_with_build_fixers(session, ["dzil", "authordeps"], fixers)
-        for entry in lines:
-            yield "build", PerlModuleRequirement(entry.strip())
+        if os.path.exists(os.path.join(self.path, "dist.ini")):
+            lines = run_with_build_fixers(session, ["dzil", "authordeps"], fixers)
+            for entry in lines:
+                yield "build", PerlModuleRequirement(entry.strip())
         if os.path.exists(os.path.join(os.path.dirname(self.path), "cpanfile")):
             yield from _declared_deps_from_cpanfile(session, fixers)
 
@@ -978,7 +1037,9 @@ class RunTests(BuildSystem):
 
 def _read_cpanfile(session, args, kind, fixers):
     for line in run_with_build_fixers(session, ["cpanfile-dump"] + args, fixers):
-        yield kind, PerlModuleRequirement(line)
+        line = line.strip()
+        if line:
+            yield kind, PerlModuleRequirement(line)
 
 
 def _declared_deps_from_cpanfile(session, fixers):
@@ -986,17 +1047,46 @@ def _declared_deps_from_cpanfile(session, fixers):
     yield from _read_cpanfile(session, ["--test"], "test", fixers)
 
 
-class Make(BuildSystem):
+def _declared_deps_from_meta_yml(f):
+    # See http://module-build.sourceforge.net/META-spec-v1.4.html for
+    # the specification of the format.
+    import ruamel.yaml
+    import ruamel.yaml.reader
 
-    name = "make"
+    try:
+        data = ruamel.yaml.load(f, ruamel.yaml.SafeLoader)
+    except ruamel.yaml.reader.ReaderError as e:
+        warnings.warn("Unable to parse META.yml: %s" % e)
+        return
+    for require in data.get("requires", None) or []:
+        yield "core", PerlModuleRequirement(require)
+    for require in data.get("build_requires", None) or []:
+        yield "build", PerlModuleRequirement(require)
+    for require in data.get("configure_requires", None) or []:
+        yield "build", PerlModuleRequirement(require)
+    # TODO(jelmer): recommends
+
+
+class Make(BuildSystem):
 
     def __init__(self, path):
         self.path = path
+        if os.path.exists(os.path.join(path, 'Makefile.PL')):
+            self.name = 'makefile.pl'
+        elif os.path.exists(os.path.join(path, 'Makefile.am')):
+            self.name = 'automake'
+        elif any([os.path.exists(os.path.join(path, n))
+                 for n in ['configure.ac', 'configure.in', 'autogen.sh']]):
+            self.name = 'autoconf'
+        elif os.path.exists(os.path.join(path, "CMakeLists.txt")):
+            self.name = 'cmake'
+        else:
+            self.name = "make"
 
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.path)
 
-    def setup(self, session, resolver, fixers):
+    def setup(self, session, resolver, fixers, prefix=None):
         def makefile_exists():
             return any(
                 [session.exists(p) for p in ["Makefile", "GNUmakefile", "makefile"]]
@@ -1025,54 +1115,87 @@ class Make(BuildSystem):
                 run_with_build_fixers(session, ["autoreconf", "-i"], fixers)
 
         if not makefile_exists() and session.exists("configure"):
-            run_with_build_fixers(session, ["./configure"], fixers)
+            extra_args = []
+            if prefix is not None:
+                extra_args.append('--prefix=%s' % prefix)
+            run_with_build_fixers(session, ["./configure"] + extra_args, fixers)
 
         if not makefile_exists() and any(
             [n.name.endswith(".pro") for n in session.scandir(".")]
         ):
             run_with_build_fixers(session, ["qmake"], fixers)
 
+        if not makefile_exists() and session.exists('CMakeLists.txt'):
+            session.mkdir('build')
+            run_with_build_fixers(session, ["cmake", '..'], fixers, cwd='build')
+
     def build(self, session, resolver, fixers):
         self.setup(session, resolver, fixers)
-        run_with_build_fixers(session, ["make", "all"], fixers)
+        self._run_make(session, ["all"], fixers)
 
     def clean(self, session, resolver, fixers):
         self.setup(session, resolver, fixers)
-        run_with_build_fixers(session, ["make", "clean"], fixers)
+        self._run_make(session, ["clean"], fixers)
+
+    def _run_make(self, session, args, fixers, prefix=None):
+        def _wants_configure(line):
+            if line.startswith("Run ./configure"):
+                return True
+            if line == "Please run ./configure first":
+                return True
+            if line.startswith("Project not configured"):
+                return True
+            if line.startswith("The project was not configured"):
+                return True
+            return False
+        if session.exists('build'):
+            cwd = 'build'
+        else:
+            cwd = None
+        try:
+            run_with_build_fixers(session, ["make"] + args, fixers, cwd=cwd)
+        except UnidentifiedError as e:
+            if len(e.lines) < 5 and any([_wants_configure(line) for line in e.lines]):
+                extra_args = []
+                if prefix is not None:
+                    extra_args.append("--prefix=%s" % prefix)
+                run_with_build_fixers(session, ["./configure"] + extra_args, fixers)
+                run_with_build_fixers(session, ["make"] + args, fixers)
+            elif (
+                "Reconfigure the source tree "
+                "(via './config' or 'perl Configure'), please."
+            ) in e.lines:
+                run_with_build_fixers(session, ["./config"], fixers)
+                run_with_build_fixers(session, ["make"] + args, fixers)
+            else:
+                raise
 
     def test(self, session, resolver, fixers):
         self.setup(session, resolver, fixers)
-        run_with_build_fixers(session, ["make", "check"], fixers)
+        self._run_make(session, ["check"], fixers)
 
     def install(self, session, resolver, fixers, install_target):
-        self.setup(session, resolver, fixers)
-        run_with_build_fixers(session, ["make", "install"], fixers)
+        self.setup(session, resolver, fixers, prefix=install_target.prefix)
+        self._run_make(session, ["install"], fixers, prefix=install_target.prefix)
 
     def dist(self, session, resolver, fixers, target_directory, quiet=False):
         self.setup(session, resolver, fixers)
         with DistCatcher.default(session.external_path(".")) as dc:
             try:
-                run_with_build_fixers(session, ["make", "dist"], fixers)
+                self._run_make(session, ["dist"], fixers)
             except UnidentifiedError as e:
                 if "make: *** No rule to make target 'dist'.  Stop." in e.lines:
                     raise NotImplementedError
                 elif "make[1]: *** No rule to make target 'dist'. Stop." in e.lines:
                     raise NotImplementedError
-                elif (
-                    "Reconfigure the source tree "
-                    "(via './config' or 'perl Configure'), please."
-                ) in e.lines:
-                    run_with_build_fixers(session, ["./config"], fixers)
-                    run_with_build_fixers(session, ["make", "dist"], fixers)
+                elif "ninja: error: unknown target 'dist', did you mean 'dino'?" in e.lines:
+                    raise NotImplementedError
                 elif (
                     "Please try running 'make manifest' and then run "
                     "'make dist' again." in e.lines
                 ):
                     run_with_build_fixers(session, ["make", "manifest"], fixers)
                     run_with_build_fixers(session, ["make", "dist"], fixers)
-                elif "Please run ./configure first" in e.lines:
-                    run_with_build_fixers(session, ["./configure"], fixers)
-                    run_with_build_fixers(session, ["make", "dist"], fixers)
                 elif any(
                     [
                         re.match(
@@ -1106,20 +1229,9 @@ class Make(BuildSystem):
         something = False
         # TODO(jelmer): Split out the perl-specific stuff?
         if os.path.exists(os.path.join(self.path, "META.yml")):
-            # See http://module-build.sourceforge.net/META-spec-v1.4.html for
-            # the specification of the format.
-            import ruamel.yaml
-            import ruamel.yaml.reader
-
             with open(os.path.join(self.path, "META.yml"), "rb") as f:
-                try:
-                    data = ruamel.yaml.load(f, ruamel.yaml.SafeLoader)
-                except ruamel.yaml.reader.ReaderError as e:
-                    warnings.warn("Unable to parse META.yml: %s" % e)
-                    return
-                for require in data.get("requires", []):
-                    yield "build", PerlModuleRequirement(require)
-                something = True
+                yield from _declared_deps_from_meta_yml(f)
+            something = True
         if os.path.exists(os.path.join(self.path, "cpanfile")):
             yield from _declared_deps_from_cpanfile(session, fixers)
             something = True
@@ -1204,6 +1316,7 @@ def _parse_go_mod(f):
     while line:
         parts = line.strip().split(" ")
         if not parts or parts == [""]:
+            line = readline()
             continue
         if len(parts) == 2 and parts[1] == "(":
             line = readline()
@@ -1319,10 +1432,11 @@ class Maven(BuildSystem):
         deps_tag = root.find("dependencies")
         if deps_tag:
             for dep in deps_tag.findall("dependency"):
+                version_tag = dep.find("version")
                 yield "core", MavenArtifactRequirement(
                     dep.find("groupId").text,
                     dep.find("artifactId").text,
-                    dep.find("version").text,
+                    version_tag.text if version_tag else None,
                 )
 
 
@@ -1393,6 +1507,7 @@ class PerlBuildTiny(BuildSystem):
 
     def __init__(self, path):
         self.path = path
+        self.minilla = os.path.exists(os.path.join(self.path, "minil.toml"))
 
     def __repr__(self):
         return "%s(%r)" % (type(self).__name__, self.path)
@@ -1402,7 +1517,10 @@ class PerlBuildTiny(BuildSystem):
 
     def test(self, session, resolver, fixers):
         self.setup(session, fixers)
-        run_with_build_fixers(session, ["./Build", "test"], fixers)
+        if self.minilla:
+            run_with_build_fixers(session, ["minil", "test"], fixers)
+        else:
+            run_with_build_fixers(session, ["./Build", "test"], fixers)
 
     def build(self, session, resolver, fixers):
         self.setup(session, fixers)
@@ -1412,9 +1530,53 @@ class PerlBuildTiny(BuildSystem):
         self.setup(session, fixers)
         run_with_build_fixers(session, ["./Build", "clean"], fixers)
 
+    def dist(self, session, resolver, fixers, target_directory, quiet=False):
+        self.setup(session, fixers)
+        with DistCatcher([session.external_path('.')]) as dc:
+            if self.minilla:
+                # minil seems to return 0 even if it didn't produce a tarball :(
+                run_with_build_fixers(
+                    session, ["minil", "dist"], fixers,
+                    check_success=lambda retcode, lines: bool(dc.find_files()))
+            else:
+                try:
+                    run_with_build_fixers(session, ["./Build", "dist"], fixers)
+                except UnidentifiedError as e:
+                    if "Can't find dist packages without a MANIFEST file" in e.lines:
+                        run_with_build_fixers(session, ["./Build", "manifest"], fixers)
+                        run_with_build_fixers(session, ["./Build", "dist"], fixers)
+                    elif "No such action 'dist'" in e.lines:
+                        raise NotImplementedError
+                    else:
+                        raise
+        return dc.copy_single(target_directory)
+
     def install(self, session, resolver, fixers, install_target):
         self.setup(session, fixers)
-        run_with_build_fixers(session, ["./Build", "install"], fixers)
+        if self.minilla:
+            run_with_build_fixers(session, ["minil", "install"], fixers)
+        else:
+            run_with_build_fixers(session, ["./Build", "install"], fixers)
+
+    def get_declared_dependencies(self, session, fixers=None):
+        self.setup(session, fixers)
+        if self.minilla:
+            pass  # Minilla doesn't seem to have a way to just regenerate the metadata :(
+        else:
+            try:
+                run_with_build_fixers(session, ["./Build", "distmeta"], fixers)
+            except UnidentifiedError as e:
+                if "No such action 'distmeta'" in e.lines:
+                    pass
+                if "Do not run distmeta. Install Minilla and `minil install` instead." in e.lines:
+                    self.minilla = True
+                else:
+                    raise
+        try:
+            with open(os.path.join(self.path, 'META.yml'), 'r') as f:
+                yield from _declared_deps_from_meta_yml(f)
+        except FileNotFoundError:
+            pass
 
     @classmethod
     def probe(cls, path):
@@ -1428,8 +1590,8 @@ BUILDSYSTEM_CLSES = [
     SetupPy,
     Npm,
     Waf,
-    Cargo,
     Meson,
+    Cargo,
     Cabal,
     Gradle,
     Maven,
@@ -1439,6 +1601,7 @@ BUILDSYSTEM_CLSES = [
     Golang,
     R,
     Octave,
+    Bazel,
     # Make is intentionally at the end of the list.
     Make,
     Composer,
