@@ -82,6 +82,15 @@ class PythonPackageRequirement(Requirement):
         return p.returncode == 0
 
 
+class LatexPackageRequirement(Requirement):
+
+    def __init__(self, package: str):
+        self.package = package
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.package)
+
+
 class PhpPackageRequirement(Requirement):
     def __init__(
         self,
@@ -168,7 +177,11 @@ class VagueDependencyRequirement(Requirement):
             from .resolver.apt import AptRequirement
 
             yield AptRequirement.simple(self.name.lower(), minimum_version=self.minimum_version)
-            yield AptRequirement.simple('lib%s-dev' % self.name.lower(), minimum_version=self.minimum_version)
+            if self.name.lower().startswith('lib'):
+                devname = '%s-dev' % self.name.lower()
+            else:
+                devname = 'lib%s-dev' % self.name.lower()
+            yield AptRequirement.simple(devname, minimum_version=self.minimum_version)
 
     def met(self, session):
         for x in self.expand():
@@ -192,6 +205,40 @@ class NodePackageRequirement(Requirement):
         return "%s(%r)" % (type(self).__name__, self.package)
 
 
+class PerlPreDeclaredRequirement(Requirement):
+
+    name: str
+
+    # TODO(jelmer): Can we obtain this information elsewhere?
+    KNOWN_MODULES = {
+        'auto_set_repository': 'Module::Install::Repository',
+        'author_tests': 'Module::Install::AuthorTests',
+        'recursive_author_tests': 'Module::Install::AuthorTests',
+        'author_requires': 'Module::Install::AuthorRequires',
+        'readme_from': 'Module::Install::ReadmeFromPod',
+        'catalyst': 'Module::Install::Catalyst',
+        'githubmeta': 'Module::Install::GithubMeta',
+        'use_ppport': 'Module::Install::XSUtil',
+        'pod_from': 'Module::Install::PodFromEuclid',
+        'write_doap_changes': 'Module::Install::DOAPChangeSets',
+        'use_test_base': 'Module::Install::TestBase',
+        'jsonmeta': 'Module::Install::JSONMETA',
+        'extra_tests': 'Module::Install::ExtraTests',
+        'auto_set_bugtracker': 'Module::Install::Bugtracker',
+        }
+
+    def __init__(self, name):
+        super(PerlPreDeclaredRequirement, self).__init__("perl-predeclared")
+        self.name = name
+
+    def lookup_module(self):
+        module = self.KNOWN_MODULES[self.name]
+        return PerlModuleRequirement(module=module)
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.name)
+
+
 class NodeModuleRequirement(Requirement):
 
     module: str
@@ -246,6 +293,10 @@ class PkgConfigRequirement(Requirement):
         self.module = module
         self.minimum_version = minimum_version
 
+    def __repr__(self):
+        return "%s(%r, minimum_version=%r)" % (
+            type(self).__name__, self.module, self.minimum_version)
+
 
 class PathRequirement(Requirement):
 
@@ -255,6 +306,9 @@ class PathRequirement(Requirement):
         super(PathRequirement, self).__init__("path")
         self.path = path
 
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.path)
+
 
 class CHeaderRequirement(Requirement):
 
@@ -264,6 +318,9 @@ class CHeaderRequirement(Requirement):
         super(CHeaderRequirement, self).__init__("c-header")
         self.header = header
 
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.header)
+
 
 class JavaScriptRuntimeRequirement(Requirement):
     def __init__(self):
@@ -415,6 +472,17 @@ class LibraryRequirement(Requirement):
         self.library = library
 
 
+class StaticLibraryRequirement(Requirement):
+
+    library: str
+    filename: str
+
+    def __init__(self, library: str, filename: str):
+        super(StaticLibraryRequirement, self).__init__("static-lib")
+        self.library = library
+        self.filename = filename
+
+
 class RubyFileRequirement(Requirement):
 
     filename: str
@@ -587,6 +655,11 @@ class LibtoolRequirement(Requirement):
         super(LibtoolRequirement, self).__init__("libtool")
 
 
+class IntrospectionTypelibRequirement(Requirement):
+    def __init__(self, library):
+        self.library = library
+
+
 class PythonModuleRequirement(Requirement):
 
     module: str
@@ -619,3 +692,17 @@ class PythonModuleRequirement(Requirement):
         )
         p.communicate()
         return p.returncode == 0
+
+    def __repr__(self):
+        return "%s(%r, python_version=%r, minimum_version=%r)" % (
+            type(self).__name__, self.module, self.python_version,
+            self.minimum_version)
+
+
+class BoostComponentRequirement(Requirement):
+
+    name: str
+
+    def __init__(self, name):
+        super(BoostComponentRequirement, self).__init__("boost-component")
+        self.name = name
